"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sleep = exports.resolvablePromise = exports.awaitLater = exports.ResourceStatus = void 0;
exports.podsForNode = podsForNode;
exports.findSuffix = findSuffix;
exports.quantityToScalar = quantityToScalar;
exports.totalCPUForContainer = totalCPUForContainer;
exports.totalMemoryForContainer = totalMemoryForContainer;
exports.totalCPU = totalCPU;
exports.totalMemory = totalMemory;
exports.add = add;
exports.containerTotalForResource = containerTotalForResource;
exports.totalForResource = totalForResource;
exports.generateTmpFileName = generateTmpFileName;
const os_1 = require("os");
const crypto_1 = require("crypto");
const fs_1 = require("fs");
async function podsForNode(api, nodeName) {
    const allPods = await api.listPodForAllNamespaces();
    return allPods.body.items.filter((pod) => pod.spec.nodeName === nodeName);
}
function findSuffix(quantity) {
    let ix = quantity.length - 1;
    while (ix >= 0 && !/[\.0-9]/.test(quantity.charAt(ix))) {
        ix--;
    }
    return ix === -1 ? '' : quantity.substring(ix + 1);
}
function quantityToScalar(quantity) {
    if (!quantity) {
        return 0;
    }
    const suffix = findSuffix(quantity);
    if (suffix === '') {
        const num = Number(quantity).valueOf();
        if (isNaN(num)) {
            throw new Error('Unknown quantity ' + quantity);
        }
        return num;
    }
    switch (suffix) {
        case 'n':
            return Number(quantity.slice(0, quantity.length - 1)).valueOf() / 1000000000;
        case 'u':
            return Number(quantity.slice(0, quantity.length - 1)).valueOf() / 1000000;
        case 'm':
            return Number(quantity.slice(0, quantity.length - 1)).valueOf() / 1000.0;
        case 'k':
            return BigInt(quantity.slice(0, quantity.length - 1)) * BigInt(1000);
        case 'M':
            return BigInt(quantity.slice(0, quantity.length - 1)) * BigInt(1000 * 1000);
        case 'G':
            return BigInt(quantity.slice(0, quantity.length - 1)) * BigInt(1000 * 1000 * 1000);
        case 'T':
            return BigInt(quantity.slice(0, quantity.length - 1)) * BigInt(1000 * 1000 * 1000) * BigInt(1000);
        case 'P':
            return (BigInt(quantity.slice(0, quantity.length - 1)) *
                BigInt(1000 * 1000 * 1000) *
                BigInt(1000 * 1000));
        case 'E':
            return (BigInt(quantity.slice(0, quantity.length - 1)) *
                BigInt(1000 * 1000 * 1000) *
                BigInt(1000 * 1000 * 1000));
        case 'Ki':
            return BigInt(quantity.slice(0, quantity.length - 2)) * BigInt(1024);
        case 'Mi':
            return BigInt(quantity.slice(0, quantity.length - 2)) * BigInt(1024 * 1024);
        case 'Gi':
            return BigInt(quantity.slice(0, quantity.length - 2)) * BigInt(1024 * 1024 * 1024);
        case 'Ti':
            return BigInt(quantity.slice(0, quantity.length - 2)) * BigInt(1024 * 1024 * 1024) * BigInt(1024);
        case 'Pi':
            return (BigInt(quantity.slice(0, quantity.length - 2)) *
                BigInt(1024 * 1024 * 1024) *
                BigInt(1024 * 1024));
        case 'Ei':
            return (BigInt(quantity.slice(0, quantity.length - 2)) *
                BigInt(1024 * 1024 * 1024) *
                BigInt(1024 * 1024 * 1024));
        default:
            throw new Error(`Unknown suffix: ${suffix}`);
    }
}
class ResourceStatus {
    constructor(request, limit, resourceType) {
        this.request = request;
        this.limit = limit;
        this.resourceType = resourceType;
    }
}
exports.ResourceStatus = ResourceStatus;
function totalCPUForContainer(container) {
    return containerTotalForResource(container, 'cpu');
}
function totalMemoryForContainer(container) {
    return containerTotalForResource(container, 'memory');
}
function totalCPU(pod) {
    return totalForResource(pod, 'cpu');
}
function totalMemory(pod) {
    return totalForResource(pod, 'memory');
}
function add(n1, n2) {
    if (typeof n1 === 'number' && typeof n2 === 'number') {
        return n1 + n2;
    }
    if (typeof n1 === 'number') {
        return BigInt(Math.round(n1)) + BigInt(n2);
    }
    else if (typeof n2 === 'number') {
        return BigInt(n1) + BigInt(Math.round(n2));
    }
    return BigInt(n1) + BigInt(n2);
}
function containerTotalForResource(container, resource) {
    let reqTotal = 0;
    let limitTotal = 0;
    if (container.resources) {
        if (container.resources.requests) {
            reqTotal = add(reqTotal, quantityToScalar(container.resources.requests[resource]));
        }
        if (container.resources.limits) {
            limitTotal = add(limitTotal, quantityToScalar(container.resources.limits[resource]));
        }
    }
    return new ResourceStatus(reqTotal, limitTotal, resource);
}
function totalForResource(pod, resource) {
    let reqTotal = 0;
    let limitTotal = 0;
    pod.spec.containers.forEach((container) => {
        const containerTotal = containerTotalForResource(container, resource);
        reqTotal = add(reqTotal, containerTotal.request);
        limitTotal = add(limitTotal, containerTotal.limit);
    });
    return new ResourceStatus(reqTotal, limitTotal, resource);
}
async function generateTmpFileName() {
    let tmpFileName;
    let i = 0;
    do {
        tmpFileName = `${(0, os_1.tmpdir)()}/${(0, crypto_1.randomUUID)()}`;
        try {
            await fs_1.promises.access(tmpFileName, fs_1.constants.W_OK);
            console.warn('Tmp file already exists');
        }
        catch (err) {
            return tmpFileName;
        }
        i++;
    } while (i < 10);
    throw new Error('Cannot generate tmp file name');
}
/**
 * If you don't want to or can't await a promise right await and need to store
 * it in a variable to await it later, wrap it with this function.
 * Otherwise, if the promise rejects before you await it, it will cause an
 * unhandled promise rejection which can cause process termination (in node).
 */
const awaitLater = (p) => {
    p.catch(() => { }); // tslint:disable-line:no-empty
    return p;
};
exports.awaitLater = awaitLater;
const resolvablePromise = () => {
    let resolve;
    let reject;
    const promise = (0, exports.awaitLater)(new Promise((res, rej) => {
        resolve = res;
        reject = rej;
    }));
    promise.resolve = resolve;
    promise.reject = reject;
    return promise;
};
exports.resolvablePromise = resolvablePromise;
const sleep = (ms) => {
    return new Promise((resolve) => setTimeout(resolve));
};
exports.sleep = sleep;
//# sourceMappingURL=util.js.map